import { WebGLRenderer, WebGLRendererParameters, Camera, Scene } from 'three';
import { SceneGLTF } from './GLTFLoader/plugin';
export * as Cannon from 'cannon-es';
export { default as TencentGLTFLoader } from './GLTFLoader';
export { default as TencentParticle } from './particle/Particle';
export interface TencentRendererParameters extends WebGLRendererParameters {
    /** 设置场景的宽度 */
    width?: number;
    /** 设置场景的高度 */
    height?: number;
    /** 设置设备像素比 */
    dpr?: number;
}
export interface Tencent3DParameters {
    /** 模型文件的路径 */
    src: string;
    /** 自动运行 */
    autoPlay?: boolean;
    /** 自动运行所有动画 */
    autoPlayAnimation?: boolean;
    /** debug 模式 */
    debug?: boolean;
    /** 加载模型的监听回调 */
    onProgress?: (event: ProgressEvent) => void;
    /** [`THREE.WebGLRenderer`参数](https://threejs.org/docs/index.html?q=webGL#api/en/renderers/WebGLRenderer) */
    rendererParameters?: TencentRendererParameters;
}
/**
 * 基于[`three.js (v147)`](https://threejs.org/)二次开发的一款3D引擎，支持[`Blender 3.4`](https://www.blender.org/)下几乎所有的eevee引擎[`ShaderNode`](https://docs.blender.org/manual/zh-hans/3.4/render/shader_nodes/index.html)。
 * ```ts
// npm install three @types/three cannon-es cannon-es cannon-es-debugger tencent3d
import Tencent3D from 'tencent3d';
const game = new Tencent3D({
    src: './untitled.glb',
    onProgress(ev) {
        console.log(ev.loaded / ev.total);
    },
    rendererParameters: {
        canvas: document.getElementById('app'),
    },
});
game.then((gltf) => {
    console.log(gltf);
});
 * ```
 */
export default class Tencent3D extends Promise<SceneGLTF> {
    #private;
    /** @ignore */
    static get [Symbol.species](): PromiseConstructor;
    /** @ignore */
    get [Symbol.toStringTag](): string;
    /** @readonly 模型数据 */
    gltf: SceneGLTF;
    /** @readonly [`THREE.WebGLRenderer`](https://threejs.org/docs/index.html?q=webGL#api/en/renderers/WebGLRenderer) */
    renderer: WebGLRenderer;
    /** @readonly canvas的宽高以及定位 */
    stageBounding: DOMRect;
    constructor(parameters: Tencent3DParameters);
    /** 初始化renderer */
    private initRenderer;
    /** 初始化场景 */
    private initScene;
    /** 初始化动画 */
    private initAnimation;
    /** 初始化摄像机控制器参数 */
    private initControlSetting;
    /** 初始化摄像机控制器 */
    private initCameraControls;
    /** 设置活动场景 */
    setActiveScene(scene: Scene): Promise<void>;
    /**
     * 设置激活相机
     * @param camera  - 激活的相机
     * @param force   - 针对`camera.visable=false`的摄像机，是否强制设置该摄像机为激活摄像机
     */
    setActiveCamera(camera: Camera, force?: boolean): Promise<void>;
    /** @ignore @readonly 帧回调 */
    private playCallBack?;
    /**
     * 运行场景
     * @param CallBack        - 每一帧的回调函数
     * @param activeScene     - 设置激活场景
     * @param activeCamera    - 设置激活摄像机
     *
     */
    play(CallBack?: Function, activeScene?: Scene, activeCamera?: Camera): Promise<void>;
    /** 暂停 */
    pause(): Promise<void>;
}
