import ShaderNode, { NodeSocket, ShaderNodeParams, SocketType, SocketTypeAll } from './shaderNode/ShaderNode';
import { WebGLRenderer, Scene, Camera, BufferGeometry, Mesh, Group, BufferAttribute, Uniform } from 'three';
import ShaderNodeMaterial from './ShaderNodeMaterial';
export interface NodeLink<T> {
    fromNode: T;
    toNode: T;
    fromSocket: number;
    toSocket: number;
}
export interface ShaderNodeTreeParams {
    nodes: ShaderNodeParams<SocketType>[];
    links: NodeLink<number>[];
}
declare const symbolDefaultUniform: unique symbol;
declare const symbolLinks: unique symbol;
/** Node tree consisting of linked nodes used for materials (and other shading data-blocks) */
export default class ShaderNodeTree {
    #private;
    private static getActiveOutput;
    /** 是否需要灯光 */
    needLights: boolean;
    /** 是否需要清漆法线 */
    needClearcoat: boolean;
    /** 是否需要丝绒光泽 */
    needSheen: boolean;
    readonly nodes: ShaderNode[];
    /** @ignore */
    private readonly [symbolLinks];
    /** 顶点着色器用到的shaderLink */
    readonly vertexLinks: NodeLink<ShaderNode>[];
    /** 顶点着色器用到的shaderLink */
    readonly fragmentLinks: NodeLink<ShaderNode>[];
    /** 顶点着色器用到的shaderNode */
    get vertexNodes(): ShaderNode[];
    /** 片元着色器用到的shaderNode */
    get fragmentNodes(): ShaderNode[];
    /** 曲线节点的数据 */
    readonly curves: BufferAttribute[];
    /** 纹理数据 */
    readonly textureMap: Map<string, Uniform> | Map<Uniform, string>;
    /** @ignore shaderNode的默认参数 */
    private [symbolDefaultUniform];
    constructor(params: ShaderNodeTreeParams);
    /** 生成曲线纹理 */
    private createCurverTexture;
    /** 顶点着色器中声明的变量 */
    vertexVariable: Set<NodeSocket>;
    /** 获取顶点着色器代码 */
    get vertexShader(): string;
    /** 片元着色器中声明的变量 */
    fragmentVariable: Set<NodeSocket>;
    /** 获取片元着色器代码 */
    get fragmentShader(): string;
    /** Index of the active output */
    private get activeOutput();
    /** Return active shader output node for the specified target */
    private get activeOutputNode();
    /** 对nodes进行tree-shaking和重新排序 */
    private optimizeNodes;
    /** 对links进行tree-shaking和重新排序 */
    private optimizeLinks;
    private getShaderLinks;
    setUniform(key: string, value: SocketType): void;
    getUniform(key: string): SocketTypeAll | null;
    getUniformSocket(key: string): NodeSocket | null;
    /** 着色器节点的内置参数 */
    get defaultUniform(): {
        [key: string]: SocketType;
    };
    /** 着色器节点内置参数的struct代码 */
    get defaultUniformStruct(): string;
    /** 缓存key 相同key可共用同一个着色器代码 */
    get cacheKey(): string;
    /** 销毁 */
    dispose(): void;
    /** 更新的uniform标记 避免每帧中shaderNode重复更新 */
    needsUpdateList: {
        [key: string]: any;
    };
    /**
     * @event
     * 编译回调
     */
    onBeforeCompile(shader: ShaderNodeMaterial, renderer: WebGLRenderer): void;
    /**
     * @event
     * 在每次渲染材质时调用（当有多个mesh引用同一材质时会运行多次）
     */
    onBeforeRender(renderer: WebGLRenderer, scene: Scene, camera: Camera, material: ShaderNodeMaterial): void;
    /**
     * @event
     * 在每次Event Loop时调用
     */
    onBeforeObjectRender(renderer: WebGLRenderer, scene: Scene, camera: Camera, object: Mesh, geometry: BufferGeometry, material: ShaderNodeMaterial, group: Group): void;
}
export {};
