/** eslint-disable @typescript-eslint/no-unused-vars */
import { BufferGeometry, Camera, Group, Mesh, Scene, WebGLRenderer, Texture, Vector2, Vector3, Vector4, Matrix3, Matrix4, Quaternion, Color } from 'three';
import Action from '../../utils/Action';
import ShaderNodeMaterial from '../ShaderNodeMaterial';
import ShaderNodeTree from '../ShaderNodeTree';
export declare type Vec2 = [number, number];
export declare type Vec3 = [number, number, number];
export declare type Vec4 = [number, number, number, number];
export declare type Mat3_flat = [
    number,
    number,
    number,
    number,
    number,
    number,
    number,
    number,
    number
];
export declare type Mat4_flat = [
    number,
    number,
    number,
    number,
    number,
    number,
    number,
    number,
    number,
    number,
    number,
    number,
    number,
    number,
    number,
    number
];
export declare type Mat3 = [
    [
        number,
        number,
        number
    ],
    [
        number,
        number,
        number
    ],
    [
        number,
        number,
        number
    ]
];
export declare type Mat4 = [
    [
        number,
        number,
        number,
        number
    ],
    [
        number,
        number,
        number,
        number
    ],
    [
        number,
        number,
        number,
        number
    ],
    [
        number,
        number,
        number,
        number
    ]
];
export declare type SocketType = boolean | number | Vec2 | Vec3 | Vec4 | Mat3_flat | Mat4_flat | Mat3 | Mat4 | Vector2 | Vector3 | Vector4 | Color | Quaternion | Matrix3 | Matrix4 | Texture;
export declare type SocketTypeAll = SocketType | string | (() => SocketType);
export interface ShaderNodeParams<T> {
    /** 输入节点默认参数 */
    inputs: T[] | undefined | null;
    /** 输出节点默认参数 */
    outputs: T[] | undefined | null;
    /** 名字 */
    name: string;
    /** 类型 */
    type: string;
    /** 是否跳过 */
    mute?: boolean;
    [key: string]: any;
}
declare const defaultUniform: unique symbol;
declare const nodeType: unique symbol;
/** Material shader node */
export default class ShaderNode implements ShaderNodeParams<NodeSocket> {
    /** 节点类型 */
    static type: string;
    /** 节点索引 */
    get index(): number;
    /** @ignore */
    private [nodeType];
    /** 实例化节点类型 */
    get type(): string;
    /** 节点的输入块 */
    inputs: ShaderNodeParams<NodeSocket>['inputs'];
    /** 节点的输出块 */
    outputs: ShaderNodeParams<NodeSocket>['outputs'];
    /** 节点名字 */
    name: ShaderNodeParams<NodeSocket>['name'];
    /** 节点是否不可用 */
    mute: ShaderNodeParams<NodeSocket>['mute'];
    /** 节点所在的节点树 */
    protected readonly nodeTree: ShaderNodeTree;
    constructor(params: ShaderNodeParams<SocketType>, nodeTree: ShaderNodeTree, type?: string);
    /** @ignore */
    private [defaultUniform];
    /** @ignore */
    protected get __defaultUniform(): Map<string, NodeSocket>;
    /** @ignore */
    protected set __defaultUniform(uniform: Map<string, NodeSocket>);
    /** @ignore */
    protected readonly customUniform: Map<string, NodeSocket>;
    /** @ignore 内置其他参数 （这些参数不在uniform struct 之中），比如传入texture等 */
    protected readonly customParams: Set<string>;
    /** @ignore 动作表达式的参数块 */
    private actionSockets;
    /** @ignore 默认参数 */
    get defaultUniform(): Map<string, NodeSocket>;
    /** @ignore 过滤不必要的输入节点 */
    private filterInputSocket;
    /** @ignore 过滤不必要的输出节点 */
    private filterOutputSocket;
    /** 销毁 */
    dispose(): void;
    /**
     * @event
     *  在编译shader程序之前立即执行的回调。此函数使用shader源码作为参数。用于修改内置材质。
     *
     * */
    onBeforeCompile(shader: ShaderNodeMaterial, renderer: WebGLRenderer): void;
    /**
     * @event
     * 在渲染之前直接执行回调函数
     *
     * */
    onBeforeRender(renderer: WebGLRenderer, scene: Scene, camera: Camera, material: ShaderNodeMaterial): void;
    /**
     * @event
     * 在渲染物体时执行回调（比`onBeforeRender`更频繁）
     */
    onBeforeObjectRender(renderer: WebGLRenderer, scene: Scene, camera: Camera, object: Mesh, geometry: BufferGeometry, material: ShaderNodeMaterial, group: Group): void;
    /** @ignore 节点的shader声明函数 */
    get parsShader(): string;
    /**
     * @ignore
     * 节点的shader调用函数
     * @param shader_type 0:vertexShader   1:fragmentShader
     */
    shader(registryData: Set<NodeSocket>, shader_type?: number): string;
}
declare type NodeSocketVector = 'vec2' | 'vec3' | 'vec4';
declare type NodeSocketMatrix = 'mat2' | 'mat3' | 'mat4';
declare type NodeSocketType = 'Closure' | 'sampler2D' | 'bool' | 'int' | 'float' | NodeSocketVector | NodeSocketMatrix;
declare const symbolType: unique symbol;
/** 节点的接口 */
export declare class NodeSocket {
    value: SocketTypeAll;
    readonly index: number;
    /** 是否是uniform */
    isUniform: boolean;
    /** 是否是动作表达式 */
    isScript: boolean;
    script?: Action;
    name: string;
    /** @ignore */
    private [symbolType];
    get type(): NodeSocketType;
    /** 节点接口的关联数据 */
    links: {
        node: ShaderNode;
        socket?: number;
        input?: number;
        output?: number;
    }[];
    /** 是否和其他接口进行关联 */
    get enabled(): boolean;
    constructor(value: SocketTypeAll, index?: number, type?: NodeSocketType);
    /** 设置接口的值 */
    setValue(value: SocketTypeAll, type?: NodeSocketType): void;
    /** 建立关联 */
    addLink<T>(node: T, socket: number): void;
}
export declare class InputSocket extends NodeSocket {
    readonly links: {
        node: ShaderNode;
        output: number;
    }[];
    addLink<T>(node: T): void;
}
export declare class OutputSocket extends NodeSocket {
    isUniform: boolean;
    readonly links: {
        node: ShaderNode;
        input: number;
    }[];
    addLink<T>(node: T): void;
}
export {};
