/** eslint-disable @typescript-eslint/naming-convention */
import { Points, BufferGeometry, WebGLRenderer, Scene, Camera, ShaderMaterial, Texture, Color, Mesh, Vector3, Line, Box3, Sphere } from 'three';
import ShaderNodeMaterial from '../material/ShaderNodeMaterial';
interface ParticleParams {
    /** 未诞生 */
    show_unborn?: boolean;
    /** 消逝 */
    show_dead?: boolean;
    /** 发射源 */
    emit_object?: Line | Mesh | Points;
    /** 发射位置(基于顶点\面\体) */
    emit_from?: 'FACE' | 'VERT' | 'VOLUME';
    /** 随机顺序 */
    emit_random?: boolean;
    /** 均匀分布 (面/体) */
    even_distribution?: boolean;
    /** 发射偏移抖动 */
    emit_jitter?: number;
    /** 总粒子数 默认: 16 */
    count?: number;
    /** 生命周期 默认: 1 */
    lifetime?: number;
    /** 生命周期次数 默认: -1(永久) */
    lifetime_count?: number;
    /** 每个生命周期的起始时间 */
    time_start?: number;
    /** 粒子发射的延迟时间 */
    time_delay?: number;
    /** 起始时间随机性 */
    time_start_random?: number;
    /** 生命周期随机性 */
    lifetime_random?: number;
    /** 质量 默认: 1 */
    mass?: number;
    /** 粒子尺寸与质量相乘 */
    multiply_size_mass?: boolean;
    /** 粒子大小 默认: 1 */
    size?: number;
    /** 缩放随机性 */
    size_random?: number;
    /** 实例物体 */
    instance_object?: (Line | Mesh | Points)[];
    /** 速度 */
    velocity?: {
        /** 法向 默认: 1 */
        normal_factor?: number;
        /** 切向 */
        tangent_factor?: number;
        /** 物体对齐 */
        align_factor?: Vector3;
        /** 物体速度 */
        object_factor?: number;
        /** 随机 */
        factor_random?: number;
    };
    /** 旋转 */
    rotation?: {
        /** 随机朝向 */
        factor_random?: number;
        /** 角速度 */
        velocity_factor?: number;
    };
    /** 力场 */
    forces?: {
        /** 布朗 */
        brownian?: number;
        /** 空气阻力 */
        air_drag?: number;
        /** 阻尼 */
        damping?: number;
        /** 重力 */
        gravity?: number;
        /** 全部 */
        all?: number;
        /** 常力 */
        force?: number;
        /** 涡流 */
        vortex?: number;
        /** 磁力 */
        magnetic?: number;
        /** 谐振 */
        harmonic?: number;
        /** 电荷 */
        charge?: number;
        /** 兰纳琼斯分子力 */
        lennardjones?: number;
        /** 风力 */
        wind?: number;
        /** 曲线引导 */
        curve_guide?: number;
        /** 纹理 */
        texture?: number;
        /** 紊流 */
        turbulence?: number;
        /** 拖拽 */
        drag?: number;
        /** 群簇 */
        boid?: number;
    };
}
/** Particle geometry */
declare class ParticleGeometry extends BufferGeometry {
    #private;
    private static mixSetting;
    particleInfo: ParticleParams;
    private get particleInfoUniform();
    private get particleInfoStruct();
    /** GPU 粒子数量 */
    private get GPUCount();
    /** GPU 纹理尺寸 */
    private get GPUWidth();
    private gpuCompute;
    private gpuInfo;
    private gpuLocation;
    private gpuVelocity;
    /** 启用开关，false时不运行粒子运动 */
    enabled: boolean;
    boundingBox: Box3;
    boundingSphere: Sphere;
    constructor(particleSetting: ParticleParams);
    private init;
    private setTextureData;
    private getTriangleArea;
    private getFormatTriangleIndex;
    private initEmitData;
    private initInfoData;
    private initLocationData;
    private initVelocityData;
    private initGPU;
    private initParticleUV;
    get locationTexture(): Texture | null;
    get velocityTexture(): Texture | null;
    get infoTexture(): Texture | null;
    private performanceTime;
    private get time();
    private set time(value);
    private get deltaTime();
    private set deltaTime(value);
    /**
     * @ignore 发射粒子
     * @param lifeCount 发射次数 -1代表持续发射粒子 0代表不发射粒子
     */
    run(lifeCount: number): void;
    /**
     * @ignore
     * 渲染粒子
     */
    render(renderer: WebGLRenderer, scene: Scene, camera: Camera, object: Particle, geometry: ParticleGeometry, material: ParticleMaterial | ShaderNodeMaterial): void;
}
declare class ParticleMaterial extends ShaderMaterial {
    isPointsMaterial: boolean;
    size: number;
    color: Color;
    map: null;
    alphaMap: null;
    sizeAttenuation: boolean;
    transparent: boolean;
    customProgramCacheKey(): string;
    constructor(object: Particle);
    render(renderer: WebGLRenderer, scene: Scene, camera: Camera, object: Particle): void;
}
/** Particle mesh */
export default class Particle extends Points {
    geometry: ParticleGeometry;
    material: ParticleMaterial | ShaderNodeMaterial;
    /** @ignore */
    get morphTargetDictionary(): {
        [key: string]: number;
    } | {
        [key: string]: number;
    } | {
        [key: string]: number;
    } | undefined;
    /** @ignore */
    get morphTargetInfluences(): number[] | undefined;
    /**
     * 发射粒子
     * @param count 发射次数 -1代表持续发射粒子 0代表不发射粒子
     */
    run(count: number): void;
    constructor(params: ParticleParams, material?: ParticleMaterial | ShaderNodeMaterial);
}
export {};
